import 'package:crab_test_package_001/data/words_list.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';

import 'friends_words_cell.dart';

class FriendsWords extends StatefulWidget {
  static const double ThisWidth = 30;
  static const double ThisPadding = 10;

  final void Function(int intIndex) dragSelectedCallback;
  final void Function() dragUnselectedCallback;

  const FriendsWords(
      {Key? key,
      required this.dragSelectedCallback,
      required this.dragUnselectedCallback})
      : super(key: key);

  @override
  _FriendsWordsState createState() => _FriendsWordsState();
}

class _FriendsWordsState extends State<FriendsWords> {
  Color _backgroundColor = const Color.fromRGBO(255, 255, 255, 0.3);
  Color _textColor = Colors.black;

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onVerticalDragStart: (DragStartDetails details) {
        setState(() {
          _backgroundColor = const Color.fromRGBO(1, 1, 1, 0.3);
          _textColor = Colors.white;
        });

        int intIndex = _getWord(details.globalPosition);
        widget.dragSelectedCallback(intIndex);
      },
      onVerticalDragUpdate: (DragUpdateDetails details) {
        int intIndex = _getWord(details.globalPosition);
        widget.dragSelectedCallback(intIndex);
      },
      onVerticalDragEnd: (DragEndDetails details) {
        setState(() {
          _backgroundColor = const Color.fromRGBO(255, 255, 255, 0.3);
          _textColor = Colors.black;
        });

        widget.dragUnselectedCallback();
      },
      onVerticalDragCancel: () {
        setState(() {
          _backgroundColor = const Color.fromRGBO(255, 255, 255, 0.3);
          _textColor = Colors.black;
        });

        widget.dragUnselectedCallback();
      },
      child: Container(
        alignment: Alignment.center,
        padding: const EdgeInsets.only(
            top: FriendsWords.ThisPadding, bottom: FriendsWords.ThisPadding),
        width: FriendsWords.ThisWidth,
        height: _getHeight(),
        child: ListView.builder(
          itemBuilder: (BuildContext context, int index) {
            return FriendsWordsCell(
              words: listWords[index],
              textColor: _textColor,
            );
          },
          itemCount: listWords.length,
          physics: const NeverScrollableScrollPhysics(),
        ),
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(6),
          color: _backgroundColor,
        ),
      ),
    );
  }

  double _getHeight() {
    return (listWords.length * FriendsWordsCell.CellHeight) +
        (FriendsWords.ThisPadding * 2);
  }

  int _getWord(Offset globalPosition) {
    RenderBox box = context.findRenderObject()! as RenderBox;
    Offset localPosition = box.globalToLocal(globalPosition);
    int intIndex = ((localPosition.dy - FriendsWords.ThisPadding) ~/
            FriendsWordsCell.CellHeight)
        .clamp(0, listWords.length - 1);

    return intIndex;
  }
}
